


#---------------------------------------------< Functions >------------------------------------------------------------#

# creates CSV used for boxplotting
def createCSV(name, data, plot_type, path, trimmed):
	df = read_csv(StringIO("".join(data)), sep="\t")
	name_groups = re.search(r"(?P<samplename>.*)_(?P<read>(R1|R2)).*", name)
	filename = path + '/' + plot_type.replace(" ", "_") + ".csv"

	if exists(filename):
		summary = read_csv (filename, header = None, sep = ",")
	else:
		summary = DataFrame()

	if plot_type == "Sequence Length Distribution":
		try:
			df["#Length"] = DataFrame(df["#Length"].str.split("-", expand=True), dtype=int).mean(axis=1)
		except:
			# Length is already of type int
			pass

	if name_groups:
		df = concat([ Series([name_groups.group("samplename")]* len(df.index), name = "Sample") ,
		              Series([trimmed ]* len(df.index), name="Trimmed"),
		              Series([name_groups.group("read") ]* len(df.index), name="Read"),
		              df ],axis =1)
	else:
		df = concat([Series([name] * len(df.index), name="Sample"),
		             Series([trimmed] * len(df.index), name="type"),
		             Series([""] * len(df.index), name="read"),
		             df],axis=1)
	if len(summary) !=0:
		summary.columns = df.columns

	summary = concat([summary, df], axis=0, ignore_index = True)
	summary_path = open(filename, 'w')
	summary.to_csv(summary_path, header = False, index = False)
	summary_path.close()


# convert result type to color (adapted to color blindness)
def get_color(value):
	if (value == "pass"):
		return "green"
	elif (value == "fail"):
		return "red"
	else:
		return "orange"

def get_fastqc_results(fastqc_path, outdir , type):
	skip = ["Basic Statistics", "Kmer Content"]
	store_data = ['Sequence Length Distribution', 'Per sequence quality scores', 'Per sequence GC content',
	              'Adapter Content', 'Overrepresented sequences']
	total_seq = 0
	overrepr_count = 0
	adapter_content = []

	fastqc_results = OrderedDict()
	data = []
	store = False

	for fastqc in fastqc_path:
		sample = OrderedDict()
		name = basename(fastqc).replace("_fastqc","")
		sample["img"]= OrderedDict()
		if getsize(fastqc) != 0:
			with open(join(fastqc, "fastqc_data.txt"), "r") as fastqc_data:
				for line in iter(fastqc_data):
					if line.startswith("Total Sequences"):
						sample["Total Sequences"] = int(line.strip().split("\t")[-1])
						total_seq += int(line.strip().split("\t")[-1])
					elif line.startswith('>>END_MODULE'):
						if len(data) > 0:
							if key[0] in store_data:
								if key[0] == 'Adapter Content':
									ac = max(read_csv(StringIO("".join(data)), sep="\t", index_col=0).max().round(2))
									adapter_content.append(ac)
									sample["%Adapter content"] = ac

								elif key[0] == "Overrepresented sequences":
									ors= read_csv(StringIO("".join(data)), sep="\t")["Count"].sum()
									overrepr_count += ors
									sample["%Overrepr sequences"] =int(ors)
								else:
									createCSV(name, data, key[0], outdir, type)
							data = []
							store = False
					elif line.startswith('>>'):
						key = line.split("\t")
						key[0] = key[0].replace(">>", "")
						if key[0] in store_data:
							store = True
						if not key[0] in skip:
							img = {}
							img["color"] = get_color(key[1].replace("\n",""))
							try:
								img["base64" ] = to_base64(join(fastqc, "Images", parameter["FastQC"][key[0]]))
								img["path"] = join(fastqc, "Images", parameter["FastQC"][key[0]])
								sample["img"][key[0]] =  img
							except:
								pass #img["base64"] = ""

					if store and not line.startswith(">>"):
						data.extend([line])
			fastqc_results[name] = sample
	try:
		adapter = round(sum(adapter_content)/len(adapter_content) ,2)
	except:
		adapter = 0
	if total_seq == 0:
		overrepr = 0
	else:
		overrepr = round(100*overrepr_count / total_seq ,2)

	return  fastqc_results, int(total_seq), overrepr , adapter

#--------------------------------------------< RULES >-----------------------------------------------------------------#

#def get_reads(wildcards):
#	if exists(sample_dict[wildcards.sample]):
#		return sample_dict[wildcards.sample]

# Run FastQC on raw data
rule fastqc_raw:
	input:
		lambda wildcards: sample_dict[wildcards.sample]
	output:
		zip= fastqc_path + "/raw/{sample}_fastqc.zip",
		html = temp(fastqc_path + "/raw/{sample}_fastqc.html"),
		folder = temp(fastqc_path + "/raw/{sample}_fastqc")
	message:
		"Run FastQC on raw data."
	threads:
		max_threads
	log:
		log_path + "/logfile.fastqc.txt"
	run:
		shell("fastqc {input} -o {path}/raw/ --extract -t {threads} >>  {log} 2>&1 ", path = fastqc_path )
		if "{path}/raw/{name}_fastqc".format(name =  get_name(str(input)), path = fastqc_path) != str(output.folder):
			shell("mv {path}/raw/{name}_fastqc {output.folder}" , name =  get_name(str(input)), path = fastqc_path )
			shell("mv {path}/raw/{name}_fastqc.html {output.html}" , name =  get_name(str(input)), path = fastqc_path)
			shell("mv {path}/raw/{name}_fastqc.zip {output.zip}" , name  =  get_name(str(input)), path = fastqc_path)


rule fastqc_trimmed:
	input:
		trimming_path + "/{sample}.fastq.gz"
	output:
		fastqc_path + "/trimmed/{sample}_fastqc.zip",
		temp(fastqc_path + "/trimmed/{sample}_fastqc.html"),
		folder = temp(fastqc_path + "/trimmed/{sample}_fastqc")
	threads:
		max_threads
	log:
		log_path + "/logfile.trimmed.fastqc.txt"
		#temp(log_path + "/{sample}.fastqc.trimmed.log")
	message:
		"Run FastQC for trimmed data."
	shell:
		"if [ `zcat '{input}' | head -n 1 | wc -c ` -eq 0 ];then touch {output};else fastqc {input} -o $(dirname {output.folder}) --extract -t {threads} >>  {log} 2>&1;  fi "

