def plot_data_by_cycle(metrics, x, y, title, output, alt_ylabel, caption ):
	df = metrics.df[[x, "%s_A" % y, "%s_C" % y, "%s_T" % y, "%s_G" % y]]
	df.columns = [x.replace(y + "_", "") for x in df.columns]
	df = df.groupby(x).mean()
	y = alt_ylabel
	df.columns.name = y
	df.index.name = x
	df.plot(title=title + " - %s" % df.columns.name, legend=True)
	plt.savefig(output)
	plt.close()
	return {"title" : title + " - %s" % df.columns.name,
	        "caption" : caption,
	        "base64" : to_base64(output) }

def plot_data_by_lane(metrics, codes, x, y, group, title, output, caption ):
	df = metrics.df[metrics.df.code.isin(codes.keys())]
	df = df[["code", "lane", "value"]]
	for key in codes.keys():
		df.code = df.code.replace(key, codes[key])
	df.index = MultiIndex.from_tuples(list(zip(df["code"], df["lane"])))
	df = df[df.value > 0].append(df[df.value == 0].drop_duplicates())
	df.columns = [group, x, y]
	ax = sns.boxplot(x=x, y=y, hue=group, data=df).set_title(title)
	plt.savefig(output)
	plt.close()
	return {"title" : title,
	        "caption" : caption,
	        "base64" : to_base64(output)
	        }

if cmd_input["sav"]:
	sav_results =  data_path + "/sav.json"

	rule plot_sav:
		input:
			cmd_input["sav"]
		output:
			#unpack(sav_results)
			pdf = main_path + "/SAV.pdf",
			plots = expand(data_path + "/{img}.png", img = ["data_by_cycle_base","data_by_cycle_fwhm","data_by_cycle_intensity",
		        "data_by_lane_phasing","data_by_lane_prephasing", "data_by_lane_cluster", "qscore_distr" , "qscore_heatmap"]),
			json = sav_results
		log:
			log_path + "/sav_report.log"
		run:
			import xmltodict
			shell("Rscript --vanilla {path}/Rscripts/sav.R {input} {outfolder}", path = config["QCumber_path"], outfolder = data_path)
			sav_plots =[]
			sav_plots.append(
				{"title": "Data by Cycle - FWHM", "caption": "The average full width of clusters at half maximum (in pixels).",
				 "base64": to_base64(data_path + "/data_by_cycle_fwhm.png"), "filename": data_path + "/data_by_cycle_fwhm.png" })
			sav_plots.append({"title": "Data by Cycle - intensity",
			                  "caption": "This plot shows the intensity by color of the 90% percentile of the data for each cycle.",
			                  "base64": to_base64(data_path + "/data_by_cycle_intensity.png"), "filename": data_path + "/data_by_cycle_intensity.png"})
			sav_plots.append({"title": "Data by Cycle - %Base",
			                  "caption": "The percentage of clusters for which the selected base has been called.",
			                  "base64": to_base64(data_path + "/data_by_cycle_base.png"), "filename": data_path + "/data_by_cycle_base.png"})
			sav_plots.append({"title": "Data by Cycle - %>=Q30",
					          "caption": 'The percentage of bases with a quality score of 30 or higher, respectively. This chart is generated after the 25th cycle, and the values represent the current cycle.',
					          "base64": to_base64(data_path + "/qscore_q30.png"), "filename": data_path + "/qscore_q30.png"})
			sav_plots.append({"title": "Data by Lane - %Phasing",
			                  "caption": 'The percentage of molecules in a cluster for which sequencing falls behind (phasing) the current cycle within a read The graph is split out per read.',
			                  "base64": to_base64(data_path + "/data_by_lane_phasing.png"), "filename": data_path + "/data_by_lane_phasing.png"})
			sav_plots.append({"title": "Data by Lane - %Prephasing",
	                  "caption": 'The percentage of molecules in a cluster for which sequencing falls behind (phasing) the current cycle within a read The graph is split out per read.',
	                  "base64": to_base64(data_path + "/data_by_lane_prephasing.png"), "filename": data_path + "/data_by_lane_prephasing.png"})
			sav_plots.append({"title": "Data by Lane - Cluster density",
	          "caption": 'The density of clusters for each tile (in thousand per mm2)',
	          "base64": to_base64(data_path + "/data_by_lane_cluster.png"), "filename": data_path + "/data_by_lane_cluster.png"})
			sav_plots.append({"title": "QScore Heatmap",
		          "caption": 'The Q-score heat map shows the Q-score by cycle for all lanes.',
		          "base64": to_base64(data_path + "/qscore_heatmap.png"), "filename": data_path + "/qscore_heatmap.png"})
			sav_plots.append({"title": "QScore Distribution",
		          "caption": 'The Q-score distribution shows the number of reads by quality score. The quality score os cumulative for current cycle and previous cycles, and only reads that pass the quality filter are included. The Q-score is based on the Phred scale. ',
		          "base64": to_base64(data_path + "/qscore_distr.png"), "filename": data_path + "/qscore_distr.png"})


			'''import illuminate


			sav_plots = []
			myDataset = illuminate.InteropDataset(str(input))
			#qualitymetrics = myDataset.QualityMetrics()
			tilemetrics = myDataset.TileMetrics()
			corintmetrics = myDataset.CorrectedIntensityMetrics()
			extractionmetrics = myDataset.ExtractionMetrics()

			sav_plots.append(plot_data_by_cycle(extractionmetrics, "cycle", "fwhm", "Data by Cycle", data_path + "/data_by_cycle_fwhm.png",
			   "FWHM", "The average full width of clusters at half maximum (in pixels)."))

			sav_plots.append(plot_data_by_cycle(extractionmetrics, "cycle", "intensity", "Data by Cycle",
				data_path + "/data_by_cycle_intensity.png", "intensity",
				"This plot shows the intensity by color of the 90 percentage percentile of the data for each cycle."))

			sav_plots.append(plot_data_by_cycle(corintmetrics, "cycle", "num_calls", "Data by Cycle",
								data_path + "/data_by_cycle_base.png", "%Base",
								"The percentage of clusters for which the selected base has been called."))

			# Data by lane
			phasing_code = list(range(200, 200 + (myDataset.meta.read_config.__len__())*2, 2))
			sav_plots.append(plot_data_by_lane( tilemetrics,
			   dict(zip(  phasing_code, [int((x-200)/2) + 1 for x in phasing_code])),
			   "Lane", "%Phasing", "Cluster", "Data by Lane - Phasing",
			   data_path + "/data_by_lane_phasing.png",'The percentage of molecules in a cluster for which sequencing falls behind (phasing) the current cycle within a read The graph is split out per read.'))

			prephasing_code = list(range(201, 201 + (myDataset.meta.read_config.__len__())*2, 2))
			sav_plots.append(plot_data_by_lane( tilemetrics,
			   dict(zip(  prephasing_code, [int((x-201)/2) + 1 for x in prephasing_code])),
			   "Lane", "%Prephasing", "Cluster", "Data by Lane - Prephasing",
			   data_path + "/data_by_lane_prephasing.png",'The percentage of molecules in a cluster for which sequencing falls behind (phasing) the current cycle within a read The graph is split out per read.')
			 )
			cluster_codes = {100: "Raw clusters", 101: "PF clusters"}
			tilemetrics.df.value = tilemetrics.df.value/1000
			sav_plots.append(plot_data_by_lane(tilemetrics,
			  cluster_codes,
			  "Lane", "Cluster density (K/mm2)", "Cluster", "Data by Lane - Cluster density",
			  data_path + "/data_by_lane_cluster.png",'The density of clusters for each tile (in thousand per mm2)')
			 )'''
			xml = OrderedDict()
			xml["tables"] = OrderedDict()
			runinfo = xmltodict.parse( "".join(open(join(str(input), "RunInfo.xml"),"r").readlines() ))
			runinfo["RunInfo"]["Run"]["Reads"] = runinfo["RunInfo"]["Run"]["Reads"]["Read"]

			xml["tables"]["RunInfo"] = runinfo["RunInfo"]["Run"]
			try:
				runparam = xmltodict.parse( "".join(open(join(str(input),"RunParameters.xml"),"r").readlines() ))
			except:
				runparam = xmltodict.parse( "".join(open(join(str(input),"runParameters.xml"),"r").readlines() ))
			runparam['RunParameters'].pop("Reads", None)

			xml["tables"]["RunParameter"] = dict( (key, value) for (key,value) in runparam["RunParameters"].items() if not key.startswith("@xml"))
			xml["img"] = sav_plots
			json.dump(xml, open(str(output.json), "w"))

			'''env = Environment(trim_blocks=True, variable_start_string='{{~', variable_end_string="~}}")
			env.loader = FileSystemLoader(config["QCumber_path"])
			template = env.get_template("sav_report.tex")
			tmp = xml["tables"]["RunInfo"]
			tmp.pop("Reads", None)
			tmp.pop("FlowcellLayout", None)
			pdf_latex = template.render(
				tabinfo=[DataFrame.from_dict(tmp, orient = "index").to_latex()],
				plots = sav_plots )
			latex = open(str(output.tex), "w")
			latex.write(pdf_latex)
			latex.close()
			#shell( "pdflatex -interaction=nonstopmode -output-directory=$(dirname {output.pdf}) {output.tex} -shell-escape 1>&2> {log}" )
			process = subprocess.Popen("pdflatex -interaction=nonstopmode -output-directory=%s %s 1> %s" % (dirname(output.pdf), output.tex, log), shell=True)
			process.communicate()'''
